mod migration;
mod migrations;
mod program;
mod section;

pub use migration::*;
pub use migrations::create_migrations;

pub use rusqlite::Connection as DbConn;

use sprinklers_core::model::{Programs, Sections};

use eyre::Result;
use rusqlite::NO_PARAMS;

pub fn setup_db() -> Result<DbConn> {
    // let conn = DbConn::open_in_memory()?;
    let mut conn = DbConn::open("test.db")?;

    let migs = create_migrations();
    migs.apply(&mut conn)?;

    Ok(conn)
}

pub fn query_sections(conn: &DbConn) -> Result<Sections> {
    let mut statement = conn.prepare_cached(
        "SELECT s.id, s.name, s.interface_id \
        FROM sections AS s;",
    )?;
    let rows = statement.query_map(NO_PARAMS, section::from_sql)?;
    let mut sections = Sections::new();
    for row in rows {
        let section = row?;
        sections.insert(section.id, section.into());
    }
    Ok(sections)
}

pub fn query_programs(conn: &DbConn) -> Result<Programs> {
    let mut statement = conn.prepare_cached(
        "
        SELECT p.id, p.name, ps.sequence, p.enabled, p.schedule
        FROM programs AS p
        INNER JOIN program_sequences AS ps
            ON ps.program_id = p.id;",
    )?;
    let rows = statement.query_map(NO_PARAMS, program::from_sql)?;
    let mut programs = Programs::new();
    for row in rows {
        let program = row?;
        programs.insert(program.id, program.into());
    }
    Ok(programs)
}
